@@ -6,7 +6,6 @@ import #1.base.*;
 import paraselene.*;
 import paraselene.supervisor.*;
 import paraselene.tag.*;
-import paraselene.tag.attr.*;
 import paraselene.tag.form.*;
 import paraselene.tag.list.*;
 import paraselene.tag.table.*;
@@ -119,6 +118,16 @@ public abstract class SuperPage extends Page {
 		// 全ページ共通の初期化処理があれば記述して下さい。
 	}
 
+	/**
+	 * AJAXサポート機能の指定。
+	 * アクセス元が携帯である場合は、この戻り値が何であっても無効になります。
+	 * これ自体か、各ページでオーバーライドしてサポートされたい機能を指定して下さい。
+	 * @return AJAXサポート。
+	 */
+	public AjaxSupport getAjaxSupport() {
+		return AjaxSupport.NO;
+	}
+
 	/**
 	 * 入力値の検証を行う。
 	 * このメソッドが呼ばれる際には必ずセッションが発生しています。
