@@ -6,10 +6,10 @@ import #1.base.*;
 import paraselene.*;
 import paraselene.supervisor.*;
 import paraselene.tag.*;
-import paraselene.tag.attr.*;
 import paraselene.tag.form.*;
 import paraselene.tag.list.*;
 import paraselene.tag.table.*;
+import paraselene.css.*;
 
 /**
  * #3
