@@ -1,6 +1,13 @@
 // Paraselene generated this source file.
 package paraselene.mockup.output;
 
+import paraselene.*;
+import paraselene.tag.*;
+import paraselene.css.*;
+import paraselene.tag.form.*;
+import paraselene.tag.list.*;
+import paraselene.tag.table.*;
+
 
 public abstract class Readme extends paraselene.Page {
 /**
