@@ -8,11 +8,13 @@ package paraselene;
 
 import java.net.*;
 import java.io.*;
+import paraselene.tag.*;
+import paraselene.css.*;
 
 /**
- * 色を示す文字列。
+ * 色を示す文字列。スレッドセーフです。
  */
-public class Color extends Text {
+public class Color extends PlainText implements AttributeValuable, CSSValuable {
 	private static final long serialVersionUID = 1L;
 	private int red = -1, green = -1, blue = -1;
 
@@ -20,7 +22,7 @@ public class Color extends Text {
 	 * 複製の作成。
 	 * @return 複製。
 	 */
-	public HTMLPart getReplica() {
+	public synchronized HTMLPart getReplica() {
 		return new Color( red, green, blue );
 	}
 
@@ -29,14 +31,14 @@ public class Color extends Text {
 	 * @param s 文字列。
 	 */
 	public Color( String s ) {
-		setColor( s );
+		setText( s );
 	}
 
 	/**
 	 * 色の設定。
 	 * @param s 文字列。
 	 */
-	public void setColor( String s ) {
+	public void setText( String s ) {
 		if ( s == null ) {
 			super.setText( s );
 			return;
@@ -93,7 +95,7 @@ public class Color extends Text {
 	 * @param g 緑(0x00 ～ 0xff)。負数の場合、transparent となります。
 	 * @param b 青(0x00 ～ 0xff)。負数の場合、transparent となります。
 	 */
-	public void setColor( int r, int g, int b ) {
+	public synchronized void setColor( int r, int g, int b ) {
 		if ( r < 0 || g < 0 || b < 0 ) {
 			red = green = blue = -1;
 			super.setText( "transparent" );
