@@ -13,12 +13,11 @@ import java.net.*;
 import javax.servlet.http.*;
 import paraselene.tag.*;
 import paraselene.supervisor.*;
-import paraselene.tag.attr.*;
 import paraselene.tag.form.*;
 
 
 /**
- * HTMLページ。
+ * HTMLページ。スレッドセーフです。
  */
 public abstract class Page implements Serializable {
 	/**
@@ -96,6 +95,35 @@ public abstract class Page implements Serializable {
 		}
 	}
 
+	/**
+	 * AJAXサポート機能。
+	 */
+	public enum AjaxSupport {
+		/**
+		 * 無効。
+		 */
+		NO,
+		/**
+		 * ポップアップダイアログ表示。
+		 */
+		POPUP_DIALOG,
+		/**
+		 * サーバープッシュ。<br>
+		 * ポップアップダイアログ表示も可能です。<br>
+		 * これを指定する場合、クライアントはサーバーに HTTP
+		 * 接続を維持したままとなります。
+		 * クライアント数が多く見込まれるサイトでは注意して下さい。
+		 */
+		SERVER_PUSH;
+		private static final long serialVersionUID = 1L;
+	}
+
+	/**
+	 * AJAXサポート機能の指定。
+	 * @return AJAXサポート。
+	 */
+	public abstract AjaxSupport getAjaxSupport();
+
 	private static final int ERROR_NO = 403;
 
 	private HashMap<String,Tag>	tag_1_dic = new HashMap<String,Tag>();
@@ -113,7 +141,7 @@ public abstract class Page implements Serializable {
 	 * 実行状態であるか？常に true を返します。
 	 * @return true:実行中、false:モックアップ解析中。
 	 */
-	protected boolean isRuntime() {
+	public boolean isRuntime() {
 		return true;
 	}
 
@@ -123,20 +151,22 @@ public abstract class Page implements Serializable {
 	 * @param parser パースした時のバージョン。
 	 * @param date パースした時の時間。
 	 */
-	protected void addVersionComment( String parser, String date ) {
+	protected void addVersionMeta( String parser, String date ) {
 		if ( !isRuntime() )	return;
-		Tag	body = getFirstTagByType( "body" );
-		if ( body == null )	return;
-		StringBuilder	buf = new StringBuilder( " Parser(" );
+		Tag	head = getFirstTagByType( "head" );
+		if ( head == null )	return;
+		StringBuilder	buf = new StringBuilder( " Parser[" );
 		buf = buf.append( parser );
 		buf = buf.append( "," );
 		buf = buf.append( date.toString() );
-		buf = buf.append( ") Runtime(" );
-		buf = buf.append( Version.getTitle() );
+		buf = buf.append( "] Runtime[" );
+		buf = buf.append( Version.VERSION );
 		buf = buf.append( "," );
 		buf = buf.append( new Date().toString() );
-		buf = buf.append( ") " );
-		body.addHTMLPart( 0, new Comment( buf.toString() ) );
+		buf = buf.append( "] " );
+		Tag	meta = new Tag( "meta", false );
+		meta.setAttribute( new Attribute( "name", "generator" ), new Attribute( "content", buf.toString() ) );
+		head.addHTMLPart( meta );
 	}
 
 	/**
@@ -144,7 +174,7 @@ public abstract class Page implements Serializable {
 	 * @param xml true:XHTML, false:HTML。
 	 * @param doctype &lt;!DOCTYPE ～ &gt;の文字列。不要であればnull。
 	 */
-	protected void setDoctype( boolean xml, String doctype ) {
+	protected synchronized void setDoctype( boolean xml, String doctype ) {
 		xml_f = xml;
 		doctype_def = doctype;
 	}
@@ -168,7 +198,7 @@ public abstract class Page implements Serializable {
 	/**
 	 * タグの全消去。
 	 */
-	public void clear() {
+	public synchronized void clear() {
 		tag_1_dic = new HashMap<String,Tag>();
 		tag_m_dic = new HashMap<String,HashMap<String,Tag>>();
 		name_dic = new HashMap<Tag,String>();
@@ -206,8 +236,10 @@ public abstract class Page implements Serializable {
 	 */
 	public String[] getDefineNameIndex() {
 		ArrayList<String>	list = new ArrayList<String>();
-		for ( String k:	tag_m_dic.keySet() ) {
-			list.add( k );
+		synchronized( this ) {
+			for ( String k:	tag_m_dic.keySet() ) {
+				list.add( k );
+			}
 		}
 		return list.toArray( new String[0] );
 	}
@@ -218,13 +250,15 @@ public abstract class Page implements Serializable {
 	 */
 	public String[] getDefineName() {
 		ArrayList<String>	list = new ArrayList<String>();
-		for ( String k: tag_1_dic.keySet() ) {
-			list.add( k );
+		synchronized( this ) {
+			for ( String k: tag_1_dic.keySet() ) {
+				list.add( k );
+			}
 		}
 		return list.toArray( new String[0] );
 	}
 
-	private ArrayList<Form>	getForm( Tag tag, ArrayList<Form> list ) {
+	private synchronized ArrayList<Form> getForm( Tag tag, ArrayList<Form> list ) {
 		Tag[]	nest = tag.getTagArray();
 		for ( int i = 0; i < nest.length; i++ ) {
 			list = getForm( nest[i], list );
@@ -262,10 +296,9 @@ public abstract class Page implements Serializable {
 	 * 通常のコーディングであればプログラマはこのメソッドを使う事はありません。
 	 * @param req リクエスト。
 	 */
-	public void reflect( RequestParameter req ) {
+	public synchronized void reflect( RequestParameter req ) {
 		modified = new Control[0];
 		if ( getMainTag() == null )	return;
-		//if ( !req.isExistRequestItem() )	return;
 		ArrayList<Form>	form = getForm( getMainTag(), new ArrayList<Form>() );
 		int	form_cnt = form.size();
 		String	form_id = "paraselene_form_id";	// 不一致用ダミー
@@ -299,7 +332,7 @@ public abstract class Page implements Serializable {
 	 * 押下ボタンの取得。
 	 * @return 押下ボタン。無ければnull。
 	 */
-	public Button getClickedButton() {
+	public synchronized Button getClickedButton() {
 		ArrayList<Form>	form = getForm( getMainTag(), new ArrayList<Form>() );
 		int	size = form.size();
 		for ( int i = 0; i < size; i++ ) {
@@ -336,11 +369,11 @@ public abstract class Page implements Serializable {
 	 * @param name 属性名。
 	 * @param uri URI。
 	 * @return URI属性。
-	 * @exception PageException new URIAttributeで例外が発生した。
+	 * @exception PageException new URIValueで例外が発生した。
 	 */
-	public URIAttribute createURI( String name, String uri ) throws PageException {
+	public Attribute createURI( String name, String uri ) throws PageException {
 		try {
-			return new URIAttribute( name, uri, getCharset() );
+			return new Attribute( name, new URIValue( uri, getCharset() ) );
 		}
 		catch( Exception e ) {
 			throw new PageException( e );
@@ -349,35 +382,35 @@ public abstract class Page implements Serializable {
 
 	/**
 	 * 自サイト内ページへのURI生成。
-	 * URIAttribute.pageToURI()の戻り値から、this.createURI()を呼んで
+	 * URIValue.pageToURI()の戻り値から、this.createURI()を呼んで
 	 * インスタンスを返します。
 	 * @param name 属性名。
 	 * @param pid ページID。
 	 * @param fragment フラグメント。#は含めないで下さい。
 	 * 不要な場合はnullを指定して下さい。
-	 * @param query {@link paraselene.tag.attr.URIAttribute#getQuery}の戻り値と同じ形式。エスケープしないで下さい。
+	 * @param query クエリー。エスケープしないで下さい。
 	 * @return URI属性。
 	 * @exception PageException new URIAttributeで例外が発生した。
 	 */
-	public URIAttribute createPageToURI( String name, PageID pid, String fragment,
-	String ... query ) throws PageException {
-		return createURI( name, URIAttribute.pageToURI( pid, fragment, query ) );
+	public Attribute createPageToURI( String name, PageID pid, String fragment,
+	QueryItem ... query ) throws PageException {
+		return createURI( name, URIValue.pageToURI( pid, fragment, query ) );
 	}
 
 	/**
 	 * 自サイト内ページへのURI生成。ダウンロード用。
-	 * URIAttribute.pageToDownloadURI()の戻り値から、this.createURI()を呼んで
+	 * URIValue.pageToDownloadURI()の戻り値から、this.createURI()を呼んで
 	 * インスタンスを返します。
 	 * @param name 属性名。
 	 * @param pid ページID。
 	 * @param filename ブラウザに認識させるファイル名。
-	 * @param query {@link paraselene.tag.attr.URIAttribute#getQuery}の戻り値と同じ形式。エスケープしないで下さい。
+	 * @param query クエリー。エスケープしないで下さい。
 	 * @return URI属性。
 	 * @exception PageException new URIAttributeで例外が発生した。
 	 */
-	public URIAttribute createPageToDownloadURI( String name, PageID pid,
-	String filename, String ... query ) throws PageException {
-		return createURI( name, URIAttribute.pageToDownloadURI(
+	public Attribute createPageToDownloadURI( String name, PageID pid,
+	String filename, QueryItem ... query ) throws PageException {
+		return createURI( name, URIValue.pageToDownloadURI(
 			pid, filename, query )
 		);
 	}
@@ -392,12 +425,12 @@ public abstract class Page implements Serializable {
 	 * @param body URIに埋め込む文字列。
 	 * @param fragment フラグメント。#は含めないで下さい。
 	 * 不要な場合はnullを指定して下さい。
-	 * @param query {@link paraselene.tag.attr.URIAttribute#getQuery}の戻り値と同じ形式。エスケープしないで下さい。
+	 * @param query クエリー。エスケープしないで下さい。
 	 * @return URI属性。
 	 * @exception PageException 別名URIを持たないページを指定した。
 	 * new URIAttributeで例外が発生した。先頭一致URIに対して、bodyにnullを指定した。
 	 */
-	public URIAttribute createAliasToURI( String name, PageID pid, String body, String fragment, String ... query ) throws PageException {
+	public Attribute createAliasToURI( String name, PageID pid, String body, String fragment, QueryItem ... query ) throws PageException {
 		PageFactory	factory = pid.getPageFactory();
 		Page	page  = factory.getPage( pid );
 		String	alias = page.getAliasURI();
@@ -405,7 +438,7 @@ public abstract class Page implements Serializable {
 		if ( alias == null ) {
 			throw new PageException( pid.toString() + " aliasURI is null." );
 		}
-		StringBuffer	buf = new StringBuffer( alias );
+		StringBuilder	buf = new StringBuilder( alias );
 		if ( !".na".equals( alias.substring( alias.length() - 3 ) ) ) {
 			if ( body == null ) {
 				throw new PageException( "body is null." );
@@ -418,10 +451,11 @@ public abstract class Page implements Serializable {
 			buf = buf.append( fragment );
 		}
 		try {
-			URIAttribute	attr = new URIAttribute( name, buf.toString(), getCharset() );
+			URIValue	uv = new URIValue( buf.toString(), getCharset() );
 			if ( query.length > 0 ) {
-				attr.setQuery( query );
+				uv.setQuery( query );
 			}
+			Attribute	attr = new Attribute( name, uv );
 			return attr;
 		}
 		catch( Exception e ) {
@@ -466,7 +500,7 @@ public abstract class Page implements Serializable {
 	 * @param name タグのname属性値。
 	 * @return タグ。配列化されたタグならnullを返します。
 	 */
-	public Tag getTag( String name ) {
+	public synchronized Tag getTag( String name ) {
 		return tag_1_dic.get( name );
 	}
 
@@ -477,11 +511,13 @@ public abstract class Page implements Serializable {
 	 */
 	public Tag[] getAllTag( String name ) {
 		String[]	pre = name.split( "\\$" );
-		HashMap<String,Tag>	map = tag_m_dic.get( pre[0] );
-		if ( map == null )	return null;
 		ArrayList<Tag>	list = new ArrayList<Tag>();
-		for ( String k : map.keySet() ) {
-			list.add( map.get( k ) );
+		synchronized( this ) {
+			HashMap<String,Tag>	map = tag_m_dic.get( pre[0] );
+			if ( map == null )	return null;
+			for ( String k : map.keySet() ) {
+				list.add( map.get( k ) );
+			}
 		}
 		return list.toArray( new Tag[0] );
 	}
@@ -536,7 +572,7 @@ public abstract class Page implements Serializable {
 		return idx[0];
 	}
 
-	private void clearDic( String name ) {
+	private synchronized void clearDic( String name ) {
 		Tag	tag = tag_1_dic.remove( name );
 		name_dic.remove( tag );
 		String	index = getIndexName( name );
@@ -551,14 +587,16 @@ public abstract class Page implements Serializable {
 	private String extendName( Tag tag ) {
 		String	org_name = tag.getNameAttribute();
 		String	index = getIndexName( org_name );
-		for ( ; ; name_seq++ ) {
-			org_name = String.format( "%s$%d", index, name_seq );
-			if ( tag_1_dic.get( org_name ) == null )	break;
+		synchronized( this ) {
+			for ( ; ; name_seq++ ) {
+				org_name = String.format( "%s$%d", index, name_seq );
+				if ( tag_1_dic.get( org_name ) == null )	break;
+			}
 		}
 		return org_name;
 	}
 
-	private void nameTagSingle(Tag tag ) {
+	private synchronized void nameTagSingle(Tag tag ) {
 		if ( tag == null )	return;
 		String	old_name = name_dic.get( tag );
 		String	new_name = tag.getNameAttribute();
@@ -594,8 +632,8 @@ public abstract class Page implements Serializable {
 	 * タグのname検証。衝突が見つかればnameを変更する。
 	 * @param tag 検証対象。
 	 */
-	public void nameTag( Tag tag ) {
-		tag.embed = this;
+	public synchronized void nameTag( Tag tag ) {
+		tag.setPage( this );
 		String	name = tag.getNameAttribute();
 		if ( name != null ) {
 			if ( Form.FORM_ID.equals( name ) )	return;
@@ -607,7 +645,7 @@ public abstract class Page implements Serializable {
 			if ( part instanceof Tag ) {
 				nameTag( (Tag)part );
 			}
-			else	part.embed = this;
+			else	part.setPage( this );
 		}
 		ArrayList<Form>	form = getForm( getMainTag(), new ArrayList<Form>() );
 		cnt = form.size();
@@ -623,7 +661,7 @@ public abstract class Page implements Serializable {
 	 * タグエントリーの除去。
 	 * @param tag 除去対象。
 	 */
-	public void removeNameEntry( Tag tag ) {
+	public synchronized void removeNameEntry( Tag tag ) {
 		ArrayList<Form>	form = getForm( getMainTag(), new ArrayList<Form>() );
 		int cnt = form.size();
 		for ( int i = 0; i < cnt; i++ ) {
@@ -638,19 +676,19 @@ public abstract class Page implements Serializable {
 			if ( part instanceof Tag ) {
 				removeNameEntry( (Tag)part );
 			}
-			else	part.embed = null;
+			else	part.setPage( null );
 		}
 		String	name = tag.getNameAttribute();
 		if ( name == null )	return;
 		clearDic( name );
-		tag.embed = null;
+		tag.setPage( null );
 	}
 
 	/**
 	 * HTMLタグの登録。
 	 * @param tag HTMLタグ。
 	 */
-	public void setMainTag( Tag tag ) {
+	public synchronized void setMainTag( Tag tag ) {
 		contents = tag;
 		nameTag( tag );
 	}
@@ -702,7 +740,7 @@ public abstract class Page implements Serializable {
 	 * @param to 新規設定インスタンス。
 	 * @param copy_f true:内容をコピーする、false:内容のコピーは行わない。
 	 */
-	public void replaceTag( Tag from, Tag to, boolean copy_f ) {
+	public synchronized void replaceTag( Tag from, Tag to, boolean copy_f ) {
 		Tag	tag = getMainTag().getDirectInnerTag( from );
 		int	idx = tag.indexOf( from );
 		tag.removeHTMLPart( from );
@@ -725,7 +763,7 @@ public abstract class Page implements Serializable {
 	 * @return HTML
 	 */
 	public String toString() {
-		StringBuffer	buf = new StringBuffer();
+		StringBuilder	buf = new StringBuilder();
 		if ( isXML() ) {
 			buf = buf.append( "<?xml version=\"1.0\" encoding=\"" );
 			buf = buf.append( getCharset() );
