@@ -12,9 +12,12 @@ import java.io.*;
 /**
  * テキスト。出力時に一切テキストの加工を行いません。<br>
  * 入力値をタグとして認識させたい場合などの特殊な用途で使用できます。
+ * スレッドセーフです。
  */
-public class PlainText extends Text {
+public class PlainText extends HTMLPart {
 	private static final long serialVersionUID = 1L;
+	private String	text;
+
 	protected PlainText() {}
 
 	/**
@@ -22,16 +25,15 @@ public class PlainText extends Text {
 	 * @param txt テキスト。
 	 */
 	public PlainText( String txt ) {
-		super( txt, StringMode.PLAIN );
+		setText( txt );
 	}
 
 	/**
-	 * コンストラクタ。
+	 * 文字列の設定。
 	 * @param txt テキスト。
-	 * @param mode 入力文字列種別。
 	 */
-	public PlainText( String txt, StringMode mode ) {
-		super( txt, mode );
+	protected void setText( String txt ) {
+		text = txt;
 	}
 
 	/**
@@ -40,11 +42,19 @@ public class PlainText extends Text {
 	 * @return 文字列。
 	 */
 	public String toString( StringMode mode ) {
-		return super.toString( StringMode.PLAIN );
+		return text;
 	}
 
 	public HTMLPart getReplica() {
-		return new PlainText( super.toString( StringMode.PLAIN ) );
+		return new PlainText( toString( StringMode.PLAIN ) );
+	}
+
+	/**
+	 * 複製の作成。getReplica()の戻り値をキャストして返します。
+	 * @return 複製。
+	 */
+	public Valuable getValuableReplica() {
+		return (Valuable)getReplica();
 	}
 }
 
