@@ -9,13 +9,11 @@ package paraselene.dyna;
 import java.io.*;
 import java.net.*;
 import java.util.*;
-import javax.swing.text.html.parser.*;
 import javax.xml.parsers.*;
 import org.xml.sax.*;
 import paraselene.*;
 import paraselene.mockup.*;
 import paraselene.tag.*;
-import paraselene.tag.attr.*;
 import paraselene.supervisor.*;
 
 class URIEcho implements URIResolver {
@@ -58,24 +56,6 @@ public class DynamicPage extends Page {
 	private static HashMap<String, String>	simple_map = new HashMap<String, String>();
 
 	static {
-		try {
-			Cache.init();
-			Runtime.getRuntime().addShutdownHook(
-				new Thread() {
-					public void run() {
-						try {
-							Cache.save();
-						}
-						catch( Exception e ) {
-							e.printStackTrace();
-						}
-					}
-				}
-			);
-		}
-		catch( Exception e ) {
-			e.printStackTrace();
-		}
 		for ( int i = 0; i < simple.length; i++ ) {
 			simple_map.put( simple[i], simple[i] );
 		}
@@ -95,7 +75,6 @@ public class DynamicPage extends Page {
 	private Page	page = null;
 	private URIResolver	resolve = null;
 	private GrantTagProvider[]	provider = null;
-	private boolean neko_f = false;
 
 	/**
 	 * コンストラクタ。以下のメソッドの戻り値は次のようになります。
@@ -105,14 +84,11 @@ public class DynamicPage extends Page {
 	 * </UL>
 	 * この動作を変更したい場合は、
 	 * このクラスの派生クラスを作成し、上記メソッドをオーバーライドして下さい。
-	 * @param neko true:HTMLパーサーにcybernekoを使用する。<br>
-	 * false:パーサーにJava標準のものを使用する。
 	 * @param ur URI解決。
 	 * @param gtp 派生クラス置換用。不要な場合は null を指定して下さい。
 	 * 配列番号の若い方から優先的にクラス解決されます。
 	 */
-	public DynamicPage( boolean neko, URIResolver ur, GrantTagProvider[] gtp ) {
-		neko_f = neko;
+	public DynamicPage( URIResolver ur, GrantTagProvider[] gtp ) {
 		resolve = ur;
 		provider = gtp;
 	}
@@ -123,15 +99,13 @@ public class DynamicPage extends Page {
 	 * <LI>getContentType
 	 * <LI>getCharset
 	 * </UL>
-	 * @param neko true:HTMLパーサーにcybernekoを使用する。<br>
-	 * false:パーサーにJava標準のものを使用する。
 	 * @param ur URI解決。
 	 * @param p 参照ページ。
 	 * @param gtp 派生クラス置換用。不要な場合は null を指定して下さい。
 	 * 配列番号の若い方から優先的にクラス解決されます。
 	 */
-	public DynamicPage( boolean neko, URIResolver ur, Page p, GrantTagProvider[] gtp ) {
-		this( neko, ur, gtp );
+	public DynamicPage( URIResolver ur, Page p, GrantTagProvider[] gtp ) {
+		this( ur, gtp );
 		page = p;
 	}
 
@@ -147,7 +121,7 @@ public class DynamicPage extends Page {
 	private boolean isXHTML( InputStream in ) throws IOException {
 		BufferedReader	r = new BufferedReader( new InputStreamReader( in ) );
 		boolean	xml = false;
-		StringBuffer	doc = null;
+		StringBuilder	doc = null;
 		while( true ) {
 			String	line = r.readLine();
 			if ( line == null )	break;
@@ -159,7 +133,7 @@ public class DynamicPage extends Page {
 			}
 			int start = line.indexOf( START_DOCTYPE );
 			if ( start >= 0 ) {
-				doc = new StringBuffer();
+				doc = new StringBuilder();
 			}
 			if ( doc != null ) {
 				if ( start > 0 ) {
@@ -180,6 +154,8 @@ public class DynamicPage extends Page {
 		return xml;
 	}
 
+	private static final String	UNICODE = "unicode";
+
 	/**
 	 * 構築。
 	 * @param str HTML(XHTML)を表す文字列。
@@ -190,7 +166,7 @@ public class DynamicPage extends Page {
 				createXML( new InStream( str ) );
 			}
 			else {
-				createHTML( new InStream( str ), null );
+				createHTML( new InStream( str ), UNICODE );
 			}
 		}
 		catch( DynamicPageException dpe ) { throw dpe; }
@@ -202,6 +178,7 @@ public class DynamicPage extends Page {
 	 * @param f HTML(XHTML)ファイル。
 	 * @param enc ファイルの文字コード。
 	 * 内容がXHTMLであった場合無視されます。
+	 * 予測できない場合は、null を指定して下さい。
 	 */
 	public void create( File f, String enc ) throws DynamicPageException {
 		try {
@@ -216,72 +193,143 @@ public class DynamicPage extends Page {
 		catch( Exception e ) { throw new DynamicPageException( e ); }
 	}
 
+	private String getEncoding( String type ) {
+		if ( type == null )	return null;
+		String[]	part = type.split( ";" );
+		for ( int i = 0; i < part.length; i++ ) {
+			String[]	sub = part[i].split( "=" );
+			for ( int j = 0; j < sub.length; j++ ) {
+				sub[j] = sub[j].trim();
+				if ( "charset".equalsIgnoreCase( sub[j] ) ) {
+					if ( j < (sub.length - 1) ) {
+						return sub[j + 1].trim();
+					}
+				}
+			}
+		}
+		return null;
+	}
+
+	/**
+	 * 構築。
+	 * @param url HTML(XHTML)ファイルを示すURL。
+	 * @param method 取得メソッド。
+	 * @param enc ファイルの文字コード。
+	 * 内容がXHTMLであった場合無視されます。
+	 * 予測できない場合は、null を指定して下さい。
+	 */
+	public void create( URL url, RequestParameter.Method method, String enc ) throws DynamicPageException {
+		InputStream	in = null;
+		try {
+			HttpURLConnection	con = (HttpURLConnection)url.openConnection();
+			con.setInstanceFollowRedirects( true );
+			con.setRequestMethod( method == RequestParameter.Method.GET?	"GET": "POST" );
+			con.connect();
+			int	retcd = con.getResponseCode();
+			if ( (retcd / 100) != 2 )	{
+				StringBuilder	buf = new StringBuilder( Integer.toString( retcd ) );
+				buf = buf.append( " " );
+				buf = buf.append( con.getResponseMessage() );
+				throw new DynamicPageException( buf.toString() );
+			}
+			if ( enc == null ) {
+				enc = getEncoding( con.getContentType() );
+			}
+			in = new BufferedInputStream( con.getInputStream() );
+			ByteArrayOutputStream	buf = new ByteArrayOutputStream();
+			byte[]	data = new byte[4096];
+			while ( true ) {
+				int	len = in.read( data );
+				if ( len <= 0 )	break;
+				buf.write( data, 0, len );
+			}
+			in.close();
+
+			data = buf.toByteArray();
+			if ( isXHTML( new InStream( data ) ) ) {
+				createXML( new InStream( data ) );
+			}
+			else {
+				createHTML( new InStream( data ), enc );
+			}
+		}
+		catch( DynamicPageException dpe ) { throw dpe; }
+		catch( Exception e ) { throw new DynamicPageException( e ); }
+		finally {
+			try {
+				if ( in != null )	in.close();
+			}
+			catch( Exception e ){}
+		}
+	}
+
 	/**
 	 * XML構築。
 	 * @param in 入力元。処理終了時にclose()されます。
 	 * @exception DynamicPageException 処理中に例外が発生した。
 	 */
-	private void createXML( InputStream in ) throws DynamicPageException {
+	private void createXML( InStream in ) throws DynamicPageException {
 		try {
-			if ( neko_f ) {
-				createHTML( in, null );
-				return;
-			}
-			else {
-				javax.xml.parsers.SAXParserFactory	factory = javax.xml.parsers.SAXParserFactory.newInstance();
-				javax.xml.parsers.SAXParser	parser = factory.newSAXParser();
-				parser.parse( in, new Handler( this ) );
-			}
-			in.close();
-			addVersionComment( Version.getTitle(), new Date().toString() );
+			createHTML( in, null );
 		}
 		catch( Exception e ) {
+			throw new DynamicPageException( e );
+		}
+		finally {
 			try {
 				in.close();
 			}
 			catch( Exception ee ) {}
-			throw new DynamicPageException( e );
 		}
 	}
 
+	String	parse_encode = null;
+
+	/**
+	 * HTML解析時文字コードの取得。
+	 * 解析した際に使用した文字コードを返します。
+	 * @return 文字コード名。ただし、XHTMLだった時、meta タグで文字コードが
+	 * 指定されていなかった時、は null を返します。
+	 */
+	public String getParsingCharset() { return parse_encode; }
+
 	/**
 	 * HTML構築。
 	 * @param is 入力元。処理終了時にclose()されます。
 	 * @param enc エンコード指定。
 	 * @exception DynamicPageException 処理中に例外が発生した。
 	 */
-	private void createHTML( InputStream is, String enc ) throws DynamicPageException {
+	private void createHTML( InStream is, String enc ) throws DynamicPageException {
 		try {
-			if ( neko_f ) {
-				org.cyberneko.html.parsers.SAXParser	parser = new org.cyberneko.html.parsers.SAXParser();
+			org.cyberneko.html.parsers.SAXParser	parser = new org.cyberneko.html.parsers.SAXParser();
+			if ( enc != null ) {
+				parser.setProperty(
+					"http://cyberneko.org/html/properties/default-encoding", enc
+				);
+			}
+			InputSource	in = new InputSource();
+			in.setByteStream( is );
+			parser.setContentHandler( new Handler( this ) );
+			parser.parse( in );
+			if ( enc == null ) {
+				enc = getEncoding( parse_encode );
 				if ( enc != null ) {
-					parser.setProperty(
-						"http://cyberneko.org/html/properties/default-encoding", enc
-					);
+					is.reload();
+					clear();
+					createHTML( is, enc );
 				}
-				InputSource	in = new InputSource();
-				in.setByteStream( is );
-				parser.setContentHandler( new Handler( this ) );
-				parser.parse( in );
 			}
-			else {
-				Reader	in = null;
-				in = enc == null?
-					new InputStreamReader( is ):
-					new InputStreamReader( is, enc );
-				in = new BufferedReader( in );
-				ParserDelegator	pd = new ParserDelegator();
-				pd.parse( in, new CallBack( this ), true );
-				in.close();
-			}
-			addVersionComment( Version.getTitle(), new Date().toString() );
+			parse_encode = enc;
+			addVersionMeta( Version.getTitle(), new Date().toString() );
 		}
 		catch( Exception e ) {
+			throw new DynamicPageException( e );
+		}
+		finally {
 			try {
 				is.close();
 			}
 			catch( Exception ee ) {}
-			throw new DynamicPageException( e );
 		}
 	}
 
@@ -290,7 +338,7 @@ public class DynamicPage extends Page {
 	 * @return 新しいインスタンス。
 	 */
 	protected DynamicPage makePage() {
-		return new DynamicPage( neko_f, resolve, this, provider );
+		return new DynamicPage( resolve, this, provider );
 	}
 
 	/**
@@ -304,15 +352,6 @@ public class DynamicPage extends Page {
 		return TagMap.makeTag( name, attr, getCharset(), resolve, provider );
 	}
 
-	/**
-	 * コメントの生成。
-	 * @param str コメント。
-	 * @return コメント。
-	 */
-	protected Comment makeComment( String str ) {
-		return new Comment( str );
-	}
-
 	/**
 	 * テキストの生成。
 	 * @param str テキスト。
@@ -367,7 +406,7 @@ public class DynamicPage extends Page {
 			System.out.println( "を指定して下さい。" );
 			System.exit( 0 );
 		}
-		DynamicPage	page = new DynamicPage( false, new URIEcho(), null );
+		DynamicPage	page = new DynamicPage( new URIEcho(), null );
 		page.create( new File( argv[0] ), argv[1] );
 		String[]	name = page.getDefineNameIndex();
 		for ( int i = 0; i < name.length; i++ ) {
@@ -382,5 +421,7 @@ public class DynamicPage extends Page {
 		System.out.println( "正常に解析可能です" );
 		System.exit( 0 );
 	}
+
+	public AjaxSupport getAjaxSupport() { return AjaxSupport.NO; }
 }
 
