@@ -14,5 +14,8 @@ public class DynamicPageException extends Exception {
 	DynamicPageException( Throwable e ) {
 		super( e );
 	}
+	DynamicPageException( String mes ) {
+		super( mes );
+	}
 }
 
