@@ -14,21 +14,17 @@ import paraselene.*;
 import paraselene.supervisor.*;
 import paraselene.tag.*;
 import paraselene.tag.table.*;
-import paraselene.tag.attr.*;
 import paraselene.dyna.mockup.*;
 
 public class Linker extends paraselene.mockup.output.Readme implements OutputNo {
 	public String getString( int no ) {
 		int	cnt = org_html.size();
-		StringBuffer	buf = new StringBuffer();
-		boolean	json_f = Boolean.valueOf( Param.JSON.get() );
+		StringBuilder	buf = new StringBuilder();
 		switch( no ) {
 		case 1:
 			return Param.PACKAGE.get();
 		case 2:
-			if ( json_f ) {
-				buf = buf.append( "/**\n * Json\n */\nPARASELENE_JSON(-1),\n" );
-			}
+			buf = buf.append( "/**\n * Json\n */\nPARASELENE_JSON(-1),\n" );
 			for ( int i = 0; i < cnt; i++ ) {
 				PrePage	page = page_map.get( org_html.get( i ) );
 				buf = buf.append( "/**\n * " );
@@ -41,14 +37,12 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 			}
 			return buf.toString();
 		case 3:
-			if ( json_f ) {
-				buf = buf.append( "page=new " );
-				buf = buf.append( Param.PACKAGE.get() );
-				buf = buf.append( ".Json();\n" );
-				buf = buf.append( "pf.addDefine(PARASELENE_JSON,page.getClass());\n");
-				buf = buf.append( "map.put(PARASELENE_JSON.id,PARASELENE_JSON);\n" );
-				buf = buf.append( "pf.returnPage(page);\n" );
-			}
+			buf = buf.append( "page=new " );
+			buf = buf.append( Param.PACKAGE.get() );
+			buf = buf.append( ".Json();\n" );
+			buf = buf.append( "pf.addDefine(PARASELENE_JSON,page.getClass());\n");
+			buf = buf.append( "map.put(PARASELENE_JSON.id,PARASELENE_JSON);\n" );
+			buf = buf.append( "pf.returnPage(page);\n" );
 			for ( int i = 0; i < cnt; i++ ) {
 				PrePage page = page_map.get( org_html.get( i ) );
 				buf = buf.append( "page=new " );
@@ -89,8 +83,10 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 			new Attribute( "http-equiv", "Content-Type" ),
 			new Attribute( "content", "text/html; charset=UTF-8" )
 		);
-		Tag[]	head = getAllTagByType( "head" );
-		head[0].addHTMLPart( 0, meta );
+		Tag	head = getFirstTagByType( "head" );
+		if ( head != null ) {
+			head.addHTMLPart( 0, meta );
+		}
 	}
 
 	void entryPage( PrePage page ) {
@@ -113,14 +109,12 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 		Output[]    output = {
 			new paraselene.mockup.output.source.base.PageLoader(),
 			new paraselene.mockup.output.source.base.PageType(),
-			Boolean.valueOf( Param.JSON.get() )?
-				new paraselene.mockup.output.source.base.Json():
-				null,
+			new paraselene.mockup.output.source.base.Json(),
 		};
 		String[]	name = { "PageLoader.java", "PageType.java", "Json.java" };
 		for ( int i = 0; i < name.length; i++ ) {
 			if ( output[i] == null )	continue;
-			StringBuffer    out_path = new StringBuffer( Make.SRC_PATH );
+			StringBuilder    out_path = new StringBuilder( Make.SRC_PATH );
 			out_path = out_path.append( File.separator );
 			out_path = out_path.append( "base" );
 			out_path = out_path.append( File.separator );
@@ -138,7 +132,7 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 		};
 		String[]	name = { "build.xml", "web.xml" };
 		for ( int i = 0; i < name.length; i++ ) {
-			StringBuffer	buf = new StringBuffer( Param.OUT_PATH.get() );
+			StringBuilder	buf = new StringBuilder( Param.OUT_PATH.get() );
 			buf = buf.append( File.separator );
 			buf = buf.append( name[i] );
 			File	file = new File( buf.toString() );
@@ -180,7 +174,7 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 			if ( !note.isEmpty() ) {
 				Column	note_column = column[column.length - 1];
 				note_column.setValueString( note );
-				note_column.setAttribute( new ColorAttribute( "bgcolor", new Color( WebColor.YELLOW ) ) );
+				note_column.setAttribute( new Attribute( "bgcolor", new Color( WebColor.YELLOW ) ) );
 			}
 			Column	path_clm = column[column.length - 2];
 			String	path_str = path_clm.getValueString();
@@ -210,7 +204,7 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 			table.addLine( l );
 		}
 
-		StringBuffer	buf = new StringBuffer( Param.OUT_PATH.get() );
+		StringBuilder	buf = new StringBuilder( Param.OUT_PATH.get() );
 		buf = buf.append( File.separator );
 		buf = buf.append( "Readme.html" );
 		String  name = buf.toString();
@@ -259,7 +253,7 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 			list.add( part[i] );
 		}
 		int	size = list.size();
-		StringBuffer	buf = new StringBuffer();
+		StringBuilder	buf = new StringBuilder();
 		for ( int i = 0; i < size; i++ ) {
 			buf = buf.append( list.get( i ) );
 			if ( i != (size - 1) )	buf = buf.append( "/" );
@@ -284,7 +278,7 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 				uri = handle.getURI();
 				String	uri_str_new = uri.toString();
 				if ( !uri_str_new.equals( uri_str ) ) {
-					StringBuffer	note = new StringBuffer( "URIPreProcessにより " );
+					StringBuilder	note = new StringBuilder( "URIPreProcessにより " );
 					note = note.append( uri_str );
 					note = note.append( " が " );
 					note = note.append( uri_str_new );
@@ -296,28 +290,34 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 		}
 		URIHandle.URIProcess	process = handle.getURIProcess();
 		if ( process == URIHandle.URIProcess.NO_PROCESS ) {
-			StringBuffer	note = new StringBuffer( "URIPreProcessにより " );
+			StringBuilder	note = new StringBuilder( "URIPreProcessにより " );
 			note = note.append( uri_str );
 			note = note.append( " は URI解決しませんでした。" );
 			addNote( note.toString() );
+			uri_str = URLDecoder.decode( uri_str, URIValue.DEFAULT_ENC );
 			return Make.escape( uri_str );
 		}
 
-		if ( uri_str.charAt( 0 ) == '#' )	return Make.escape( uri_str );
+		if ( uri_str.charAt( 0 ) == '#' ){
+			uri_str = URLDecoder.decode( uri_str, URIValue.DEFAULT_ENC );
+			return Make.escape( uri_str );
+		}
 		if ( uri.getScheme() != null ) {
 			addNote( "絶対パスがありました(" + uri_str + ")" );
+			uri_str = URLDecoder.decode( uri_str, URIValue.DEFAULT_ENC );
 			return Make.escape( uri_str );
 		}
 		String	path = uri.getPath();
 		if ( path.charAt( 0 ) == '/' ) {
 			addNote( "ルートからのパスがありました(" + path + ")" );
+			uri_str = URLDecoder.decode( uri_str, URIValue.DEFAULT_ENC );
 			return Make.escape( uri_str );
 		}
 		uri = crt_page.getMyselfURI().resolve( uri );
 		path = fixPath( uri.getPath() );
 		PrePage	page = page_map.get( path );
 		if ( page != null ) {
-			StringBuffer	buf = new StringBuffer( "paraselene.tag.attr.URIAttribute.pageToURI(" );
+			StringBuilder	buf = new StringBuilder( "URIValue.pageToURI(" );
 			buf = buf.append( page.getEnumName( false ) );
 			buf = buf.append( "," );
 			String	frag = uri.getFragment();
@@ -325,6 +325,7 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 				buf = buf.append( "null" );
 			}
 			else {
+				frag = URLDecoder.decode( frag, URIValue.DEFAULT_ENC );
 				buf = buf.append( Make.escape( frag ) );
 			}
 			String	query = uri.getQuery();
@@ -332,7 +333,17 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 				String[]	q_part = query.split( "&" );
 				for ( int i = 0; i < q_part.length; i++ ) {
 					buf = buf.append( "," );
-					buf = buf.append( Make.escape( q_part[i] ) );
+					String[]	q = q_part[i].split( "=" );
+					for ( int j = 0; j < q.length; j++ ) {
+						q[j] = URLDecoder.decode( q[j], URIValue.DEFAULT_ENC );
+					}
+					buf = buf.append( "new QueryItem(" );
+					buf = buf.append( Make.escape( q[0] ) );
+					if ( q.length > 1 ) {
+						buf = buf.append( "," );
+						buf = buf.append( Make.escape( q[1] ) );
+					}
+					buf = buf.append( ")" );
 				}
 			}
 			buf = buf.append( ")" );
@@ -340,8 +351,8 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 			if ( port == -1 )	port = 0;
 			String	sch = uri.getScheme();
 			if ( sch != null ) {
-				StringBuffer	sub = new StringBuffer( "paraselene.tag.attr.URIAttribute.makeAbsolutePath(" );
-				sub = sub.append( "http".equals( sch )?	"paraselene.tag.attr.URIAttribute.Scheme.HTTP,":	"paraselene.tag.attr.URIAttribute.Scheme.HTTPS," );
+				StringBuilder	sub = new StringBuilder( "URIValue.makeAbsolutePath(" );
+				sub = sub.append( "http".equals( sch )?	"URIValue.Scheme.HTTP,":	"URIValue.Scheme.HTTPS," );
 				sub = sub.append( port );
 				sub = sub.append( "," );
 				sub = sub.append( buf );
@@ -351,19 +362,20 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 		}
 
 		if ( process == URIHandle.URIProcess.NOT_USE_OTHER ) {
-			StringBuffer	note = new StringBuffer( "URIPreProcessにより " );
+			StringBuilder	note = new StringBuilder( "URIPreProcessにより " );
 			note = note.append( uri_str );
 			note = note.append( " は " );
 			note = note.append( Param.OTHER.key );
 			note = note.append( " を適用しませんでした。" );
 			addNote( note.toString() );
+			uri_str = URLDecoder.decode( uri_str, URIValue.DEFAULT_ENC );
 			return Make.escape( uri_str );
 		}
 		String	ex_path = Output.toHtmlPath( path );
 		String	ex = ex_map.get( ex_path );
 		if ( ex != null ) {
 			String	str = Param.OTHER.get().trim();
-			StringBuffer	buf = new StringBuffer( str );
+			StringBuilder	buf = new StringBuilder( str );
 			if ( str.charAt( str.length() - 1 ) != '/' ) {
 				buf = buf.append( "/" );
 			}
@@ -378,9 +390,10 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 				buf = buf.append( "#" );
 				buf = buf.append( frag );
 			}
-			return Make.escape( new URI( buf.toString() ).toString() );
+			return Make.escape( URLDecoder.decode( new URI( buf.toString() ).toString(), URIValue.DEFAULT_ENC ) );
 		}
 		addNote( "警告:" + path + "が解決できませんでした。" );
+		uri_str = URLDecoder.decode( uri_str, URIValue.DEFAULT_ENC );
 		return Make.escape( uri_str );
 	}
 
@@ -402,5 +415,7 @@ public class Linker extends paraselene.mockup.output.Readme implements OutputNo
 	public void echoln( String str ) {
 		echo( str + "\n" );
 	}
+
+	public AjaxSupport getAjaxSupport() { return AjaxSupport.NO; }
 }
 
