@@ -10,19 +10,20 @@ package paraselene.mockup;
 import java.util.HashMap;
 import java.util.ArrayList;
 import java.util.Locale;
+import java.net.*;
 import paraselene.*;
 import paraselene.tag.*;
-import paraselene.tag.attr.*;
 import paraselene.tag.list.*;
 import paraselene.tag.form.*;
 import paraselene.tag.table.*;
 import paraselene.dyna.*;
+import paraselene.css.*;
 
 abstract class TagNew {
 	abstract String toString( Tag t );
 
 	static String append( String ... s ) {
-		StringBuffer	buf = new StringBuffer();
+		StringBuilder	buf = new StringBuilder();
 		for ( int i = 0; i < s.length; i++ ) {
 			buf = buf.append( s[i] );
 		}
@@ -58,7 +59,7 @@ public class TagMap {
 	private static HashMap<String,Object>	new_str = new HashMap<String,Object>();
 	private static HashMap<String,DynaTag>	tag_str = new HashMap<String,DynaTag>();
 
-	static final String TAG = "paraselene.tag.Tag";
+	static final String TAG = "Tag";
 
 	static private String getKey( Tag t ) throws TagException {
 		String	name = t.getName();
@@ -67,7 +68,7 @@ public class TagMap {
 			if ( attr == null ) {
 				throw new TagException( "type指定の無いINPUTタグがあります" );
 			}
-			Text	txt = attr.get();
+			AttributeValuable	txt = attr.get();
 			if ( txt == null ) {
 				throw new TagException( "type属性が不正です" );
 			}
@@ -128,7 +129,6 @@ public class TagMap {
 	public static Tag makeTag( String n, ArrayList<Attribute> attr, String enc, URIResolver ur, GrantTagProvider[] ex ) throws Exception {
 		GrantTag	gt = null;
 		Tag	t = new Tag( n, DynamicPage.isSimpleTag( n ) );
-		t.setAttribute( attr.toArray( new Attribute[0] ) );
 		if ( ex != null ) {
 			Attribute[]	a = attr.toArray( new Attribute[0] );
 			for ( int i = 0; i < ex.length; i++ ) {
@@ -142,15 +142,33 @@ public class TagMap {
 			Attribute	a = attr.get( i );
 			String	a_name = a.getName();
 			String	a_val = a.getString();
-			if ( URIAttribute.isURI( a_name ) ) {
+			if ( Converter.isURI( a_name ) ) {
 				a_val = ur.resolve( a_val );
 			}
-			if ( ColorAttribute.isColor( a_name ) ) {
-				a = new ColorAttribute( a_name, new Color( a_val ) );
+			if ( Converter.isColor( a_name ) ) {
+				a = new Attribute( a_name, new Color( a_val ) );
 				attr.set( i, a );
 			}
-			else if ( URIAttribute.isURI( a_name ) ) {
-				a = new URIAttribute( a_name, a_val, enc );
+			else if ( Converter.isURI( a_name ) ) {
+				if ( a_val == null )	a_val = "#";
+				else {
+					a_val = a_val.trim();
+					if ( a_val.isEmpty() )	a_val = "#";
+				}
+				try {
+					URI u = new URI( a_val );
+					a_val = URLDecoder.decode( u.toString(), enc );
+				}
+				catch( Exception e ) {}
+				a = new Attribute( a_name, new URIValue( a_val, enc ) );
+				attr.set( i, a );
+			}
+			else if ( Converter.isNumber( a_name ) ) {
+				a = new Attribute( a_name, new NumberValue( a_val ) );
+				attr.set( i, a );
+			}
+			else if ( Converter.isStyle( a_name ) ) {
+				a = new Attribute( a_name, new Style( Property.create( a_val ) ) );
 				attr.set( i, a );
 			}
 			t.setAttribute( a );
@@ -176,16 +194,16 @@ public class TagMap {
 		String	value = value_atr.getString();
 		if ( name == null || value == null )	return t;
 		if ( !ur.isParamURIName( name.toLowerCase( Locale.ENGLISH ) ) )	return t;
-		t.setAttribute( new URIAttribute( name, ur.resolve( value ), enc ) );
+		t.setAttribute( new Attribute( name, new URIValue( ur.resolve( value ), enc ) ) );
 		return t;
 	}
 
 	private static final Object[][]	dim = new Object[][]{
 	// input
-		{ "hidden", "paraselene.tag.form.Input",
+		{ "hidden", "Input",
 			new TagNew() {
 				public String toString( Tag t ) {
-					return append( "paraselene.tag.form.Input.createHidden( ", Make.escape( t.getNameAttribute() ), ",", Make.escape( t.getValueString() ), ")" );
+					return append( "Input.createHidden( ", Make.escape( t.getNameAttribute() ), ",", Make.escape( t.getValueString() ), ")" );
 				}
 			},
 			new DynaTag() {
@@ -198,72 +216,72 @@ public class TagMap {
 				}
 			}
 		},
-		{ "file", "paraselene.tag.form.UploadFile",
-			"new paraselene.tag.form.UploadFile()",
+		{ "file", "UploadFile",
+			"new UploadFile()",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new UploadFile(), attr );
 				}
 			}
 		},
-		{ "button", "paraselene.tag.form.Button",
-			"new paraselene.tag.form.Button(paraselene.tag.form.Button.Type.BUTTON)",
+		{ "button", "Button",
+			"new Button(Button.Type.BUTTON)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new Button( Button.Type.BUTTON ), attr );
 				}
 			}
 		},
-		{ "image", "paraselene.tag.form.Button",
-			"new paraselene.tag.form.Button(paraselene.tag.form.Button.Type.IMAGE)",
+		{ "image", "Button",
+			"new Button(Button.Type.IMAGE)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new Button( Button.Type.IMAGE ), attr );
 				}
 			}
 		},
-		{ "reset", "paraselene.tag.form.Button",
-			"new paraselene.tag.form.Button(paraselene.tag.form.Button.Type.RESET)",
+		{ "reset", "Button",
+			"new Button(Button.Type.RESET)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new Button( Button.Type.RESET ), attr );
 				}
 			}
 		},
-		{ "submit", "paraselene.tag.form.Button",
-			"new paraselene.tag.form.Button(paraselene.tag.form.Button.Type.SUBMIT)",
+		{ "submit", "Button",
+			"new Button(Button.Type.SUBMIT)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new Button( Button.Type.SUBMIT ), attr );
 				}
 			}
 		},
-		{ "radio", "paraselene.tag.form.CheckBox",
-			"new paraselene.tag.form.CheckBox(true)",
+		{ "radio", "CheckBox",
+			"new CheckBox(true)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new CheckBox( true ), attr );
 				}
 			}
 		},
-		{ "checkbox", "paraselene.tag.form.CheckBox",
-			"new paraselene.tag.form.CheckBox(false)",
+		{ "checkbox", "CheckBox",
+			"new CheckBox(false)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new CheckBox( false ), attr );
 				}
 			}
 		},
-		{ "password", "paraselene.tag.form.SingleTextBox",
-			"new paraselene.tag.form.SingleTextBox(true)",
+		{ "password", "SingleTextBox",
+			"new SingleTextBox(true)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new SingleTextBox( true ), attr );
 				}
 			}
 		},
-		{ "text", "paraselene.tag.form.SingleTextBox",
-			"new paraselene.tag.form.SingleTextBox(false)",
+		{ "text", "SingleTextBox",
+			"new SingleTextBox(false)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new SingleTextBox( false ), attr );
@@ -272,112 +290,112 @@ public class TagMap {
 		},
 
 	// 通常
-		{ "form", "paraselene.tag.form.Form",
-			"new paraselene.tag.form.Form()",
+		{ "form", "Form",
+			"new Form()",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new Form(), attr );
 				}
 			}
 		},
-		{ "textarea", "paraselene.tag.form.MultiTextBox",
-			"new paraselene.tag.form.MultiTextBox()",
+		{ "textarea", "MultiTextBox",
+			"new MultiTextBox()",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new MultiTextBox(), attr );
 				}
 			}
 		},
-		{ "select", "paraselene.tag.form.Select",
-			"new paraselene.tag.form.Select()",
+		{ "select", "Select",
+			"new Select()",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new Select(), attr );
 				}
 			}
 		},
-		{ "option", "paraselene.tag.form.SelectItem",
-			"new paraselene.tag.form.SelectItem()",
+		{ "option", "SelectItem",
+			"new SelectItem()",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new SelectItem(), attr );
 				}
 			}
 		},
-		{ "ul", "paraselene.tag.list.List",
-			"new paraselene.tag.list.List(paraselene.tag.list.List.Type.UNORDERED)",
+		{ "ul", "List",
+			"new List(List.Type.UNORDERED)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new List( List.Type.UNORDERED ), attr );
 				}
 			}
 		},
-		{ "ol", "paraselene.tag.list.List",
-			"new paraselene.tag.list.List(paraselene.tag.list.List.Type.ORDERED)",
+		{ "ol", "List",
+			"new List(List.Type.ORDERED)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new List( List.Type.ORDERED ), attr );
 				}
 			}
 		},
-		{ "dl", "paraselene.tag.list.List",
-			"new paraselene.tag.list.List(paraselene.tag.list.List.Type.DEFINITION)",
+		{ "dl", "List",
+			"new List(List.Type.DEFINITION)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new List( List.Type.DEFINITION ), attr );
 				}
 			}
 		},
-		{ "li", "paraselene.tag.list.ListItem",
-			"new paraselene.tag.list.ListItem(paraselene.tag.list.ListItem.Type.LIST_ITEM)",
+		{ "li", "ListItem",
+			"new ListItem(ListItem.Type.LIST_ITEM)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new ListItem( ListItem.Type.LIST_ITEM ), attr );
 				}
 			}
 		},
-		{ "dt", "paraselene.tag.list.ListItem",
-			"new paraselene.tag.list.ListItem(paraselene.tag.list.ListItem.Type.DEFINITION_TERM)",
+		{ "dt", "ListItem",
+			"new ListItem(ListItem.Type.DEFINITION_TERM)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new ListItem( ListItem.Type.DEFINITION_TERM ), attr );
 				}
 			}
 		},
-		{ "dd", "paraselene.tag.list.ListItem",
-			"new paraselene.tag.list.ListItem(paraselene.tag.list.ListItem.Type.DEFINITION_DESCRIPTION)",
+		{ "dd", "ListItem",
+			"new ListItem(ListItem.Type.DEFINITION_DESCRIPTION)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new ListItem( ListItem.Type.DEFINITION_DESCRIPTION ), attr );
 				}
 			}
 		},
-		{ "table", "paraselene.tag.table.Table",
-			"new paraselene.tag.table.Table()",
+		{ "table", "Table",
+			"new Table()",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new Table(), attr );
 				}
 			}
 		},
-		{ "tr", "paraselene.tag.table.Line",
-			"new paraselene.tag.table.Line()",
+		{ "tr", "Line",
+			"new Line()",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new Line(), attr );
 				}
 			}
 		},
-		{ "th", "paraselene.tag.table.Column",
-			"new paraselene.tag.table.Column(paraselene.tag.table.Column.Type.HEADER)",
+		{ "th", "Column",
+			"new Column(Column.Type.HEADER)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new Column( Column.Type.HEADER ), attr );
 				}
 			}
 		},
-		{ "td", "paraselene.tag.table.Column",
-			"new paraselene.tag.table.Column(paraselene.tag.table.Column.Type.DATA)",
+		{ "td", "Column",
+			"new Column(Column.Type.DATA)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
 					return setAttr( new Column( Column.Type.DATA ), attr );
@@ -395,7 +413,7 @@ public class TagMap {
 		class_str.put( TAG, TAG );
 		new_str.put( TAG, new TagNew() {
 				public String toString( Tag t ) {
-					return append( "new paraselene.tag.Tag(", Make.escape( t.getName() ), ",", Boolean.toString( t.isSimpleTag() ), ")" );
+					return append( "new Tag(", Make.escape( t.getName() ), ",", Boolean.toString( t.isSimpleTag() ), ")" );
 				}
 			}
 		);
