@@ -8,10 +8,10 @@ package paraselene.mockup;
 
 
 import java.net.*;
+import java.util.*;
 
 import paraselene.*;
 import paraselene.tag.*;
-import paraselene.tag.attr.*;
 
 
 class TagSource extends Tag implements Source {
@@ -35,15 +35,18 @@ class TagSource extends Tag implements Source {
 
 	private String getAttributeSource( Attribute a ) throws TagException {
 		String	name = a.getName();
-		Text	text = a.get();
+		AttributeValuable	text = a.get();
 		String	val = null;
-		if ( text != null )	val = text.toString();
+		if ( text != null )	val = text.toString( HTMLPart.StringMode.ATTRIBUTE );
 
 		if ( "name".equals( name ) && val != null ) {
 			val = Page.getIndexName( val );
 		}
 
-		if ( ColorAttribute.isColor( name ) ) {
+		StringBuilder	buf = new StringBuilder( "new Attribute(" );
+		buf = buf.append( Make.escape( name ) );
+		buf = buf.append( "," );
+		if ( Converter.isColor( name ) ) {
 			try {
 				if ( text != null ) {
 					val = new Color( val ).toString();
@@ -52,15 +55,33 @@ class TagSource extends Tag implements Source {
 			catch(Exception e){
 				throw new TagException( "色の指定が不正です" );
 			}
-			StringBuffer	buf = new StringBuffer( "new paraselene.tag.attr.ColorAttribute(" );
-			buf = buf.append( Make.escape( name ) );
-			buf = buf.append( ",new paraselene.Color(" );
+			buf = buf.append( "new Color(" );
 			buf = buf.append( Make.escape( val ) );
 			buf = buf.append( "))" );
 			return buf.toString();
 		}
 
-		boolean	uri_f = URIAttribute.isURI( name );
+		else if ( Converter.isNumber( name ) ) {
+			try {
+				new NumberValue( val );
+			}
+			catch(Exception e){
+				throw new TagException( "値の指定が不正です" );
+			}
+			buf = buf.append( "new NumberValue(" );
+			buf = buf.append( Make.escape( val ) );
+			buf = buf.append( "))" );
+			return buf.toString();
+		}
+
+		else if ( Converter.isStyle( name ) ) {
+			buf = buf.append( "new Style(Property.create(" );
+			buf = buf.append( Make.escape( val ) );
+			buf = buf.append( ")))" );
+			return buf.toString();
+		}
+
+		boolean	uri_f = Converter.isURI( name );
 		if ( !uri_f ) {
 			if ( "param".equals( getName() ) ) {
 				if ( "value".equalsIgnoreCase( name ) ) {
@@ -84,35 +105,26 @@ class TagSource extends Tag implements Source {
 
 		if ( uri_f ) {
 			try {
-				String	tmp = val.trim();
-				if ( tmp.isEmpty() ) {
-					val = "#";
-					Linker.readme.addNote( "空文字のパスを\"#\"に置換しました" );
-				}
 				URI	uri = new URI( val );
 				val = Linker.readme.resolvePath( uri );
 			}
 			catch(Exception e){
 				throw new TagException( "URI[" + val + "]の指定が不正です" );
 			}
-			StringBuffer	buf = new StringBuffer( "new paraselene.tag.attr.URIAttribute(" );
-			buf = buf.append( Make.escape( name ) );
-			buf = buf.append( "," );
+			buf = buf.append( "new URIValue(" );
 			buf = buf.append( val );
-			buf = buf.append( ",getCharset())" );
+			buf = buf.append( "))" );
 			return buf.toString();
 		}
 
-		StringBuffer	buf = new StringBuffer( "new paraselene.tag.attr.Attribute(" );
-		buf = buf.append( Make.escape( name ) );
-		buf = buf.append( ",");
+		buf = buf.append( "new Text(");
 		buf = buf.append( Make.escape( val ) );
-		buf = buf.append( ")" );
+		buf = buf.append( "))" );
 		return buf.toString();
 	}
 
 	private static String append( String ... s ) {
-		StringBuffer	buf = new StringBuffer();
+		StringBuilder	buf = new StringBuilder();
 		for ( int i = 0; i < s.length; i++ ) {
 			buf = buf.append( s[i] );
 		}
@@ -126,17 +138,29 @@ class TagSource extends Tag implements Source {
 		return false;
 	}
 
+	private Attribute[]	eraseAttribute( Attribute[] a ) {
+		ArrayList<Attribute>	ret = new ArrayList<Attribute>();
+		for ( int i = 0; i < a.length; i++ ) {
+			String	name = a[i].getName(), val = a[i].getString();
+			if ( val != null && "id".equals( name ) ) {
+				if ( val.indexOf( "paraselene$id$" ) >= 0 )	continue;
+			}
+			ret.add( a[i] );
+		}
+		return ret.toArray( new Attribute[0] );
+	}
+
 	public String toSource() {
-		StringBuffer	buf = null;
+		StringBuilder	buf = null;
 		try {
 			String[]	name = TagMap.getSource( this );
-			buf = new StringBuffer( name[0] );
+			buf = new StringBuilder( name[0] );
 			buf = buf.append( " " );
 			buf = buf.append( val_name );
 			buf = buf.append( "=" );
 			buf = buf.append( name[1] );
 			buf = buf.append( ";\n" );
-			Attribute[]	attr = getAllAttribute();
+			Attribute[]	attr = eraseAttribute( getAllAttribute() );
 			if ( attr.length > 0 ) {
 				buf = buf.append( val_name );
 				buf = buf.append( ".setAttribute(" );
