@@ -20,9 +20,9 @@ class TextSource extends Text implements Source {
 	}
 
 	public String toSource() {
-		StringBuffer	buf = new StringBuffer( "paraselene.Text " );
+		StringBuilder	buf = new StringBuilder( "Text " );
 		buf = buf.append( val );
-		buf = buf.append( "=new paraselene.Text(" );
+		buf = buf.append( "=new Text(" );
 		buf = buf.append( Make.escape( toString() ) );
 		buf = buf.append( ");\n" );
 		return buf.toString();
