@@ -31,7 +31,7 @@ public class KilledByTimeoutException extends Exception {
 	}
 
 	public String toString() {
-		StringBuffer	buf = new StringBuffer( super.toString() );
+		StringBuilder	buf = new StringBuilder( super.toString() );
 		int	cnt = snap.size();
 		for ( int i = 0; i < cnt; i++ ) {
 			buf = buf.append( "\n=====[job " );
