@@ -55,11 +55,6 @@ public abstract class NullPage extends Page {
 
 	public final boolean isCheckRepeatSameRequest() { return false; }
 
-}
-
-class NullPage4Alias extends NullPage {
-	public Forward input( RequestParameter req, Forward fw ) throws PageException {
-		return fw;
-	}
+	public AjaxSupport getAjaxSupport() { return AjaxSupport.NO; }
 }
 
