@@ -8,7 +8,6 @@ package paraselene.supervisor;
 
 
 import paraselene.*;
-import paraselene.tag.attr.*;
 import java.util.*;
 import java.net.*;
 import java.io.*;
@@ -51,7 +50,7 @@ class SessionData implements Serializable {
 			req.getHeader( "X-Forwarded-For" ),
 			req.getHeader( "Remote-Addr" )
 		};
-		StringBuffer	buf = new StringBuffer();
+		StringBuilder	buf = new StringBuilder();
 		for ( int i = 0; i < data.length; i++ ) {
 			if ( data[i] == null )	data[i] = "non";
 			buf = buf.append( "*" );
@@ -90,8 +89,8 @@ class SessionData implements Serializable {
 		rd.out_page = out_ret;
 		URI	uri = holder.request.getURI();
 		rd.uri = TransactionSequencer.makeRedirectURI( seq, out_ret[0].getID() );
-		String	path = URIAttribute.makeAbsolutePath(
-			URIAttribute.Scheme.getScheme( uri ), uri.getPort(), rd.uri
+		String	path = URIValue.makeAbsolutePath(
+			URIValue.Scheme.getScheme( uri ), uri.getPort(), rd.uri
 		);
 		synchronized( pending ) {
 			pending.put( out_ret[0].getID(), rd );
