@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009-2012  Akira Terasaki
+ * Copyright (c) 2009  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -9,22 +9,18 @@ package paraselene.supervisor;
 
 import paraselene.*;
 import paraselene.ajax.*;
-import paraselene.tag.attr.*;
 import java.io.*;
 import java.net.*;
 import java.util.*;
 import java.util.zip.*;
 import javax.servlet.*;
 import javax.servlet.http.*;
-import org.apache.commons.fileupload.*;
-import org.apache.commons.fileupload.disk.*;
-import org.apache.commons.fileupload.servlet.*;
 
 
 /**
- * サイト制御。
+ * サイト制御(ファイルアップロード機能なし)。
  */
-public abstract class Supervisor extends HttpServlet implements Cloneable {
+public abstract class Supervisor extends HttpServlet {
 	/**
 	 * 不正リクエスト。
 	 */
@@ -69,15 +65,11 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 		SandBox.doStart();
 	}
 
-	private static Supervisor org_sv = null;
 	/**
 	 * コンストラクタ。
 	 */
 	public Supervisor() {
 		super();
-		if ( org_sv == null ) {
-			org_sv = this;
-		}
 	}
 
 	private static String resolvePage( DataHolder holder, String p ) {
@@ -92,7 +84,7 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 				to.getPageFactory().returnPage( page );
 			}
 			if ( to != null ) {
-				StringBuffer	path = new StringBuffer(
+				StringBuilder	path = new StringBuilder(
 					TransactionSequencer.makeURI(
 						( page.isCheckRepeatSameRequest() )?
 							holder.data.seq: null,
@@ -122,7 +114,7 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 	private static String resolveHost( DataHolder holder, String path, int def_no ) {
 		try {
 			int	port = Integer.parseInt( path.substring( LINK_LEN[def_no] ) );
-			StringBuffer	buf = new StringBuffer( LINK_PROT[def_no] );
+			StringBuilder	buf = new StringBuilder( LINK_PROT[def_no] );
 			buf = buf.append( holder.req.getServerName() );
 			if ( port > 0 && port < 0xffff ) {
 				buf = buf.append( ":" );
@@ -149,11 +141,10 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 		synchronized( data_map_th ) {
 			holder = data_map_th.get( Thread.currentThread() );
 		}
-		if ( holder == null )	return URIAttribute.encode( uri, enc );
-		if ( holder.search_engine )	return URIAttribute.encode( uri, enc );
+		if ( holder == null )	return URIValue.encode( uri, enc );
+		if ( holder.search_engine )	return URIValue.encode( uri, enc );
 		String[]	part = uri.split( "/" );
 		boolean	flag = false;
-		String	head = null;
 		for ( int i = 0; i < part.length; i++ ) {
 			int	def_no;
 			for ( def_no = 0; def_no < LINK_DEF.length; def_no++ ) {
@@ -170,26 +161,18 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 			case 1:
 			case 2:
 				part[i] = resolveHost( holder, part[i], def_no );
-				part[i] = "";
 				break;
 			}
 		}
 		if ( flag ) {
-			StringBuffer	buf = new StringBuffer( part[0] );
+			StringBuilder	buf = new StringBuilder( part[0] );
 			for ( int i = 1; i < part.length; i++ ) {
-				if ( buf.length() > 0 )	buf = buf.append( "/" );
+				buf = buf.append( "/" );
 				buf = buf.append( part[i] );
 			}
 			uri = buf.toString();
 		}
-		uri = holder.response.encodeURL( URIAttribute.encode( uri, enc ) );
-		if ( head != null ) {
-			StringBuilder	buf = new StringBuilder( head );
-			buf = buf.append( "/" );
-			buf = buf.append( uri );
-			uri = buf.toString();
-		}
-		return uri;
+		return holder.response.encodeURL( URIValue.encode( uri, enc ) );
 	}
 
 	/**
@@ -240,8 +223,6 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 	 */
 	public abstract Forward onIllegalRequest( IllegalRequest ir, RequestParameter req, Page from, Page to );
 
-	private static final Page	null4alias = new NullPage4Alias();
-
 	private void doAction( HttpServletRequest request, HttpServletResponse res ) {
 		HttpSession	session = null;
 		SessionData	data = null;
@@ -283,8 +264,8 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 						fw = onIllegalRequest( IllegalRequest.SESSION_ERROR, req, path_data.from, path_data.to );
 						if ( fw != null ) {
 							session_f = fw.session_clear_f;
-							path_data.from = last_page = null;
-							path_data.to = next_page = fw.getPage( null );
+							last_page = fact.getNullPage();
+							next_page = fw.getPage( null );
 							if ( next_page == null ) {
 								fw.Action( this, res );
 								if ( session_f )	if (session != null )	session.invalidate();
@@ -294,9 +275,6 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 					}
 				}
 			}
-			else {
-				path_data.from = null4alias;
-			}
 			if ( path_data == null ) {
 				fw = new Forward( 403 );
 				fw.Action( this, res );
@@ -443,13 +421,13 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 			session.setAttribute( SESSION_DATA, data );
 		}
 		holder.myself = next_page.getID();
-		if ( !session_f && param.getMethod() == RequestParameter.Method.POST ) {
+		if ( param.getMethod() == RequestParameter.Method.POST ) {
 			data.redirect( out_ret, fw, holder ).Action( this, res );
 		}
 		else {
 			String	mime = next_page.getContentType();
 			if ( mime == null ) {
-				StringBuffer	buf = null;
+				StringBuilder	buf = null;
 				if ( next_page.isXML() ) {
 					RequestParameter.HeaderQuality[]	head = param.getHeaderWithQuality( "Accept" );
 					if ( head != null ) {
@@ -459,7 +437,7 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 								if ( f <= 0.0 )	continue;
 							}
 							if ( !XHTML.equalsIgnoreCase( head[i].getValue() ) )	continue;
-							buf = new StringBuffer( XHTML );
+							buf = new StringBuilder( XHTML );
 							break;
 						}
 					}
@@ -467,12 +445,12 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 						RequestParameter.Mobile	mob = param.judgMobile();
 						switch( mob ) {
 						case AU: case DOCOMO: case J_PHONE:
-							buf = new StringBuffer( XHTML );
+							buf = new StringBuilder( XHTML );
 							break;
 						}
 					}
 				}
-				if ( buf == null )	buf = new StringBuffer( "text/html" );
+				if ( buf == null )	buf = new StringBuilder( "text/html" );
 				buf = buf.append( "; charset=" );
 				mime = buf.append( next_page.getCharset() ).toString();
 			}
@@ -494,7 +472,7 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 				res.addHeader( "Cache-Control", "public" );
 			}
 			else {
-				//res.addHeader( "Cache-Control", "no-cache, no-store" );
+				res.addHeader( "Cache-Control", "no-cache, no-store" );
 			}
 			if ( fw != null ) {
 				int	ck_cnt = fw.out_cookie.size();
@@ -638,7 +616,7 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 		return new Page[]{ next_page, view };
 	}
 
-	private RequestParameter normalReq( HttpServletRequest req ) {
+	RequestParameter normalReq( HttpServletRequest req ) {
 		RequestParameter	ret = new RequestParameter();
 		Enumeration<?>	e = req.getParameterNames();
 		while( e.hasMoreElements() ) {
@@ -651,32 +629,6 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 		return ret;
 	}
 
-	private RequestParameter fileReq( HttpServletRequest req, PathData p )
-	throws Exception {
-		RequestParameter	ret = new RequestParameter();
-		DiskFileItemFactory	fact = new DiskFileItemFactory();
-		ServletFileUpload	upload = new ServletFileUpload( fact );
-		int	max = p.from.getUploadMaxBytes();
-		if ( max < 0 )	max = -1;
-		upload.setSizeMax( max );
-		upload.setHeaderEncoding( p.from.getCharset() );
-		List<?>	items = upload.parseRequest( req );
-		for ( Object o : items ) {
-			FileItem	item = (FileItem)o;
-			if ( item.isFormField() ) {
-				ret.addItem( item.getFieldName(), item.getString( req.getCharacterEncoding() ) );
-			}
-			else {
-				File	file = File.createTempFile( "dog", null );
-				file.deleteOnExit();
-				item.write( file );
-				String[]	path = item.getName().split( "[\\\\/]" );
-				ret.addItem( item.getFieldName(), path[path.length - 1], file, item.getContentType() );
-			}
-		}
-		return ret;
-	}
-
 	/**
 	 * 接続制限数を返す。戻り値を上回ると503を返します。
 	 * @return 0以下なら無制限。1以上なら制限する。
@@ -720,16 +672,11 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 			}
 		}
 		RequestParameter	req = null;
-		if ( !ServletFileUpload.isMultipartContent( request ) ) {
-			req = normalReq( request );
+		try {
+			req = parseRequest( request, p );
 		}
-		else {
-			try {
-				req = fileReq( request, p );
-			}
-			catch( Exception e ) {
-				return onErrorDebug( e, request, res );
-			}
+		catch( Exception e ){
+			return onErrorDebug( e, request, res );
 		}
 		req.setParam( method, request, this );
 		String[]	ua = req.getHeader( RequestParameter.UA_HEAD );
@@ -749,41 +696,44 @@ public abstract class Supervisor extends HttpServlet implements Cloneable {
 		return null;
 	}
 
+	RequestParameter parseRequest( HttpServletRequest request, PathData p ) throws Exception {
+		return normalReq( request );
+	}
+
 	protected final void doGet( HttpServletRequest req, HttpServletResponse res )  {
 		try {
-			Supervisor	sv = (Supervisor)clone();
-			sv.doMain( RequestParameter.Method.GET, req, res );
-		}
-		catch( CloneNotSupportedException ce ){}
-	}
-
-	protected final void doPost( HttpServletRequest req, HttpServletResponse res )  {
-		try{
-			Supervisor	sv = (Supervisor)clone();
-			sv.doMain( RequestParameter.Method.POST, req, res );
-		}
-		catch( CloneNotSupportedException ce ){}
-	}
-
-	private void doMain( RequestParameter.Method m, HttpServletRequest req, HttpServletResponse res )  {
-		try {
-			Forward	fw = preset( m, req, res );
+			Forward	fw = preset( RequestParameter.Method.GET, req, res );
 			if ( fw != null ) {
 				fw.Action( this, res );
 				return;
 			}
 			doAction( req, res );
-		}
-		catch( Throwable e ) {
-			onErrorDebug( e, req, res );
-		}
-		finally {
 			synchronized( data_map_th ) {
 				data_map_th.remove( Thread.currentThread() );
 				data_map_sv.remove( this );
 			}
 		}
+		catch( Exception e ) {
+			onErrorDebug( e, req, res );
+		}
 	}
 
+	protected final void doPost( HttpServletRequest req, HttpServletResponse res )  {
+		try {
+			Forward	fw = preset( RequestParameter.Method.POST, req, res );
+			if ( fw != null ) {
+				fw.Action( this, res );
+				return;
+			}
+			doAction( req, res );
+			synchronized( data_map_th ) {
+				data_map_th.remove( Thread.currentThread() );
+				data_map_sv.remove( this );
+			}
+		}
+		catch( Exception e ) {
+			onErrorDebug( e, req, res );
+		}
+	}
 }
 
