@@ -66,7 +66,7 @@ public class TransactionSequencer implements Serializable {
 	 * @return URI。
 	 */
 	public static String makeURI( TransactionSequencer seq, PageID from, PageID to ) {
-		StringBuffer	buf = new StringBuffer();
+		StringBuilder	buf = new StringBuilder();
 		if ( seq != null ) {
 			buf = buf.append( SALANE[0] );
 			buf = buf.append( "." );
@@ -89,7 +89,7 @@ public class TransactionSequencer implements Serializable {
 	}
 
 	static String makeRedirectURI( TransactionSequencer seq, PageID to ) {
-		StringBuffer	buf = new StringBuffer();
+		StringBuilder	buf = new StringBuilder();
 		buf = buf.append( SALANE[0] );
 		buf = buf.append( "." );
 		buf = buf.append( HEAD[1] );
@@ -159,7 +159,7 @@ public class TransactionSequencer implements Serializable {
 				Page	out = ret.to;
 				if ( out == null )	out = ret.redirect;
 				if ( out != null ) {
-					if ( seq != null && out.isCheckRepeatSameRequest() && !request_id ) {
+					if ( out.isCheckRepeatSameRequest() && !request_id ) {
 						return null;
 					}
 				}
