@@ -11,10 +11,10 @@ import java.io.*;
 import java.net.*;
 import paraselene.*;
 import paraselene.supervisor.*;
-import paraselene.tag.attr.*;
 
 /**
  * HTML汎用タグ。
+ * スレッドセーフです。
  */
 public class Tag extends HTMLPart {
 	private static final long serialVersionUID = 1L;
@@ -22,11 +22,54 @@ public class Tag extends HTMLPart {
 	protected HashMap<String,Attribute>	attr = new HashMap<String,Attribute>();
 	private ArrayList<HTMLPart>		doc = new ArrayList<HTMLPart>();
 	private ArrayList<Tag>		tag_only = new ArrayList<Tag>();
+	private Tag		parent = null;
 	private boolean nest_f = false;
 	private boolean visible_f = true;
 
 	private static String	VALUE = "value";
 
+	/**
+	 * 外側のタグを返す。
+	 * @return 外側のタグ。自身が最外であるならnull。
+	 */
+	public Tag getOuterTag() {
+		return parent;
+	}
+
+	private synchronized Tag[] getOverUnder() {
+		Tag[]	ret = new Tag[]{ null, null };
+		if ( parent == null )	return ret;
+		int	cnt = parent.tag_only.size();
+		for ( int i = 0; i < cnt; i++ ) {
+			if ( parent.tag_only.get( i ) == this ) {
+				if ( i > 0 ) {
+					ret[0] = parent.tag_only.get( i - 1 );
+				}
+				if ( i < cnt - 1 ) {
+					ret[1] = parent.tag_only.get( i + 1 );
+				}
+				break;
+			}
+		}
+		return ret;
+	}
+
+	/**
+	 * 同階層の真上のタグを返す。
+	 * @return 近接する真上のタグを返す。
+	 */
+	public Tag getOverTag() {
+		return getOverUnder()[0];
+	}
+
+	/**
+	 * 同階層の真下のタグを返す。
+	 * @return 近接する真下のタグを返す。
+	 */
+	public Tag getUnderTag() {
+		return getOverUnder()[1];
+	}
+
 	protected Tag() {}
 
 	private static final String[]	plain_tag = new String[] {
@@ -79,6 +122,22 @@ public class Tag extends HTMLPart {
 		visible_f = flag;
 	}
 
+	private static volatile short tag_id;
+	private static final String PARASELENE_ID = "paraselene$id$";
+
+	private void makeID() {
+		if ( getAttribute( "id" ) != null )	return;
+		long	now = 0;
+		synchronized( VALUE ){
+			now = tag_id;
+			tag_id++;
+		}
+		long	time = (now << 56) | (new Date().getTime() & 0x00ffffffffffffffL);
+		StringBuilder	buf = new StringBuilder( PARASELENE_ID );
+		buf = buf.append( Long.toString( time, Character.MAX_RADIX ) );
+		setAttribute( new Attribute( "id", buf.toString() ) );
+	}
+
 	/**
 	 * コンストラクタ。
 	 * @param n タグ名。
@@ -87,6 +146,7 @@ public class Tag extends HTMLPart {
 	public Tag( String n, boolean simple_f ) {
 		name = n.toLowerCase( Locale.ENGLISH );
 		nest_f = !simple_f;
+		if ( "body".equals( name ) )	makeID();
 	}
 
 	/**
@@ -97,7 +157,7 @@ public class Tag extends HTMLPart {
 		try {
 			Attribute	n = getAttribute( "name" );
 			if ( n == null )	return null;
-			Text	t = n.get();
+			AttributeValuable	t = n.get();
 			if ( t == null )	return null;
 			return t.toString();
 		}
@@ -112,8 +172,10 @@ public class Tag extends HTMLPart {
 	 */
 	public Attribute[] getAllAttribute() {
 		ArrayList<Attribute>	list = new ArrayList<Attribute>();
-		for ( String k : attr.keySet() ) {
-			list.add( attr.get( k ) );
+		synchronized( this ) {
+			for ( String k : attr.keySet() ) {
+				list.add( attr.get( k ) );
+			}
 		}
 		return list.toArray( new Attribute[0] );
 	}
@@ -124,7 +186,7 @@ public class Tag extends HTMLPart {
 	 * addHTMLPart()する時に配列化にされる場合があります。
 	 * @return 複製。
 	 */
-	public HTMLPart getReplica() {
+	public synchronized HTMLPart getReplica() {
 		Tag	tag = newReplica();
 		tag.visible_f = this.visible_f;
 		for ( String k : attr.keySet() ) {
@@ -158,7 +220,22 @@ public class Tag extends HTMLPart {
 	 * @param at 属性。
 	 */
 	public void setAttribute( Attribute at ) {
-		attr.put( at.getName(), at );
+		String	at_name = at.getName();
+		if ( "id".equals( at_name ) ) {
+			if ( getAttribute( "id" ) != null ) {
+				String	val = at.getString();
+				if ( val != null ) {
+					if ( val.indexOf( PARASELENE_ID ) == 0 )	return;
+				}
+			}
+		}
+		synchronized( this ) {
+			attr.put( at.getName(), at );
+		}
+		if ( "name".equals( at_name ) ) {
+			Tag	out = getOuterTag();
+			if ( out != null )	out.makeID();
+		}
 	}
 
 	/**
@@ -175,7 +252,7 @@ public class Tag extends HTMLPart {
 	 * 属性の削除。
 	 * @param at_name 属性名。
 	 */
-	public void removeAttribute( String at_name ) {
+	public synchronized void removeAttribute( String at_name ) {
 		attr.remove( at_name.toLowerCase( Locale.ENGLISH ) );
 	}
 
@@ -183,7 +260,7 @@ public class Tag extends HTMLPart {
 	 * 属性の取得。無ければnullを返す。
 	 * @param at_name 属性名。
 	 */
-	public Attribute getAttribute( String at_name ) {
+	public synchronized Attribute getAttribute( String at_name ) {
 		return attr.get( at_name.toLowerCase( Locale.ENGLISH ) );
 	}
 
@@ -201,7 +278,7 @@ public class Tag extends HTMLPart {
 	 * @param idx 追加位置。
 	 * @param d 文書。
 	 */
-	public void addHTMLPart( int idx, HTMLPart d ) {
+	public synchronized void addHTMLPart( int idx, HTMLPart d ) {
 		doc.add( idx, d );
 		if ( d instanceof Tag ) {
 			Tag tag = (Tag)d;
@@ -218,9 +295,11 @@ public class Tag extends HTMLPart {
 				}
 			}
 			tag_only.add( idx, tag );
+			tag.parent = this;
+			if ( tag.getAttribute( "name" ) != null )	makeID();
 		}
 		else {
-			d.embed = this.embed;
+			d.setPage( this.embed );
 		}
 	}
 
@@ -252,7 +331,7 @@ public class Tag extends HTMLPart {
 	 * タグで挟まれた部分全てを取り込む。そのタグ自体は取り込まない。
 	 * @param tag 取り込むタグ。
 	 */
-	public void include( Tag tag ) {
+	public synchronized void include( Tag tag ) {
 		if ( tag == null )	return;
 		int	cnt = tag.getHTMLPartCount();
 		for ( int i = 0; i < cnt; i++ ) {
@@ -265,18 +344,17 @@ public class Tag extends HTMLPart {
 	 * bodyタグの内側部分全てを取り込む。
 	 * @param page 取り込むページ。
 	 */
-	public void include( Page page ) {
-		Tag[]	tag = page.getAllTagByType( "body" );
+	public synchronized void include( Page page ) {
+		Tag	tag = page.getFirstTagByType( "body" );
 		if ( tag == null )	return;
-		if ( tag.length == 0 )	return;
-		include( tag[0] );
+		include( tag );
 	}
 
 	/**
 	 * HTML文書の取得。
 	 * @param idx 取得位置。
 	 */
-	public HTMLPart getHTMLPart( int idx ) {
+	public synchronized HTMLPart getHTMLPart( int idx ) {
 		return doc.get( idx );
 	}
 
@@ -284,7 +362,7 @@ public class Tag extends HTMLPart {
 	 * HTML文書件数の取得。
 	 * @return 件数。
 	 */
-	public int getHTMLPartCount() {
+	public synchronized int getHTMLPartCount() {
 		return doc.size();
 	}
 
@@ -292,7 +370,7 @@ public class Tag extends HTMLPart {
 	 * HTML文書の削除。
 	 * @param idx 削除位置。
 	 */
-	public void removeHTMLPart( int idx ) {
+	public synchronized void removeHTMLPart( int idx ) {
 		HTMLPart	p = doc.remove( idx );
 		if ( p == null )	return;
 		removeHTMLPart( p );
@@ -302,17 +380,18 @@ public class Tag extends HTMLPart {
 	 * HTML文書の削除。最初に登場した指定インスタンスを削除します。
 	 * @param part 削除対象。
 	 */
-	public void removeHTMLPart( HTMLPart part ) {
+	public synchronized void removeHTMLPart( HTMLPart part ) {
 		if ( part == null )	return;
 		if ( part instanceof Tag ) {
 			Tag	tag = (Tag)part;
 			tag_only.remove( tag );
+			tag.parent = null;
 			if ( embed != null ) {
 				embed.removeNameEntry( tag );
 			}
 		}
 		else {
-			part.embed = null;
+			part.setPage( null );
 		}
 		doc.remove( part );
 	}
@@ -320,7 +399,7 @@ public class Tag extends HTMLPart {
 	/**
 	 * HTML文書の全削除。
 	 */
-	public void removeHTMLPart() {
+	public synchronized void removeHTMLPart() {
 		int	cnt = doc.size();
 		for ( cnt--; cnt >= 0; cnt-- ) {
 			removeHTMLPart( cnt );
@@ -332,7 +411,7 @@ public class Tag extends HTMLPart {
 	 * removeHTMLPartと異なり、除去対象タグの内部のデータが残存します。
 	 * @param tag 除去対象。
 	 */
-	public void exclude( Tag tag ) {
+	public synchronized void exclude( Tag tag ) {
 		int	idx = indexOf( tag );
 		removeHTMLPart( tag );
 		int	cnt = tag.getHTMLPartCount();
@@ -345,7 +424,7 @@ public class Tag extends HTMLPart {
 	 * 位置を返す。最初に登場した指定インスタンスの位置を返します。
 	 * @param part 検出対象。無ければ-1を返す。
 	 */
-	public int indexOf( HTMLPart part ) {
+	public synchronized int indexOf( HTMLPart part ) {
 		if ( part == null ) return -1;
 		return doc.indexOf( part );
 	}
@@ -354,12 +433,12 @@ public class Tag extends HTMLPart {
 	 * 開始タグの文字列。
 	 * @return 開始タグ。
 	 */
-	protected String getFirstTag() {
+	protected synchronized String getFirstTag() {
 		boolean	xml = false;
 		if ( embed != null ) {
 			xml = embed.isXML();
 		}
-		StringBuffer	buf = new StringBuffer( "<" );
+		StringBuilder	buf = new StringBuilder( "<" );
 		buf = buf.append( getName() );
 		for ( String k : attr.keySet() ) {
 			buf = buf.append( " " );
@@ -377,7 +456,7 @@ public class Tag extends HTMLPart {
 	 * @return 終了タグ。
 	 */
 	protected String getLastTag() {
-		StringBuffer	buf = new StringBuffer( "</" );
+		StringBuilder	buf = new StringBuilder( "</" );
 		buf = buf.append( name );
 		buf = buf.append( ">" );
 		return buf.toString();
@@ -416,11 +495,11 @@ public class Tag extends HTMLPart {
 	 * 文書の出力。
 	 * @retrun 文書の文字列化。
 	 */
-	private String getHTMLParts() {
+	private synchronized String getHTMLParts( StringMode mode ) {
 
 		int	cnt = getHTMLPartCount();
-		StringBuffer	buf = new StringBuffer();
-		StringMode	mode = selectMode();
+		StringBuilder	buf = new StringBuilder();
+		if ( mode != StringMode.TEXTAREA )	mode = selectMode();
 		for ( int i = 0; i < cnt; i++ ) {
 			buf = buf.append( getHTMLPart( i ).toString( mode ) );
 		}
@@ -434,9 +513,9 @@ public class Tag extends HTMLPart {
 	 */
 	public String toString( StringMode mode ) {
 		if ( !visible_f )	return "";
-		StringBuffer	buf = new StringBuffer( "\n" );
+		StringBuilder	buf = new StringBuilder( "\n" );
 		buf = buf.append( getFirstTag() );
-		buf = buf.append( getHTMLParts() );
+		buf = buf.append( getHTMLParts( mode ) );
 		if ( nest_f ) {
 			buf = buf.append( getLastTag() );
 		}
@@ -468,7 +547,7 @@ public class Tag extends HTMLPart {
 	protected String getValueAttribute() {
 		Attribute	attr = getAttribute( VALUE );
 		if ( attr == null )	return null;
-		Text	text = attr.get();
+		AttributeValuable	text = attr.get();
 		if ( text == null )	return null;
 		return text.toString( StringMode.PLAIN );
 	}
@@ -483,11 +562,13 @@ public class Tag extends HTMLPart {
 	 */
 	public String getValueString() {
 		if  ( nest_f ) {
-			int	cnt = getHTMLPartCount();
-			for ( int i = 0; i < cnt; i++ ) {
-				HTMLPart	pt = getHTMLPart( i );
-				if ( pt instanceof Text ) {
-					return ((Text)pt).toString( StringMode.PLAIN );
+			synchronized( this ) {
+				int	cnt = getHTMLPartCount();
+				for ( int i = 0; i < cnt; i++ ) {
+					HTMLPart	pt = getHTMLPart( i );
+					if ( pt instanceof Text ) {
+						return ((Text)pt).toString( StringMode.PLAIN );
+					}
 				}
 			}
 			return null;
@@ -519,15 +600,17 @@ public class Tag extends HTMLPart {
 	 */
 	public void setValueString( String v ) {
 		if ( nest_f ) {
-			int	cnt = getHTMLPartCount();
-			for ( int i = 0; i < cnt; i++ ) {
-				HTMLPart	pt = getHTMLPart( i );
-				if ( pt instanceof Text ) {
-					((Text)pt).setText( v );
-					return;
+			synchronized( this ) {
+				int	cnt = getHTMLPartCount();
+				for ( int i = 0; i < cnt; i++ ) {
+					HTMLPart	pt = getHTMLPart( i );
+					if ( pt instanceof Text ) {
+						((Text)pt).setText( v, StringMode.PLAIN );
+						return;
+					}
 				}
+				addHTMLPart( new Text( v ) );
 			}
-			addHTMLPart( new Text( v ) );
 			return;
 		}
 		setValueAttribute( v );
@@ -548,7 +631,7 @@ public class Tag extends HTMLPart {
 	 * 再帰的に全てのタグを探すわけではありません。
 	 * @return タグ。無ければ0個の配列を返す。
 	 */
-	public Tag[] getTagArray() {
+	public synchronized Tag[] getTagArray() {
 		return tag_only.toArray( new Tag[0] );
 	}
 
@@ -633,14 +716,19 @@ public class Tag extends HTMLPart {
 	 */
 	public Tag getDirectInnerTag( HTMLPart target ) {
 		if ( this == target )	return null;
-		int	cnt = doc.size();
-		for ( int i = 0; i < cnt; i++ ) {
-			HTMLPart	part = doc.get( i );
-			if ( part == target )	return this;
-			if ( part instanceof Tag ) {
-				Tag	tag = ((Tag)part).getDirectInnerTag( target );
-				if ( tag != null )	return tag;
-			}
+		if ( target instanceof Tag ) {
+			return ((Tag)target).getOuterTag();
+		}
+		synchronized( this ) {
+			int	cnt = doc.size();
+			for ( int i = 0; i < cnt; i++ ) {
+				HTMLPart	part = doc.get( i );
+				if ( part == target )	return this;
+				if ( part instanceof Tag ) {
+					Tag	tag = ((Tag)part).getDirectInnerTag( target );
+					if ( tag != null )	return tag;
+				}
+			}	
 		}
 		return null;
 	}
@@ -668,7 +756,7 @@ public class Tag extends HTMLPart {
 	/**
 	 * BRタグを改行コードに置換する。
 	 */
-	public void replaceBR() {
+	public synchronized void replaceBR() {
 		int	cnt = getHTMLPartCount();
 		for ( int i = 0; i < cnt; i++ ) {
 			HTMLPart	p = getHTMLPart( i );
@@ -691,5 +779,12 @@ public class Tag extends HTMLPart {
 		}
 	}
 
+	public synchronized void setPage( Page p ) {
+		super.setPage( p );
+		Attribute[]	attr = getAllAttribute();
+		for ( int i = 0; i < attr.length; i++ ) {
+			attr[i].setPage( p );
+		}
+	}
 }
 
