@@ -14,7 +14,6 @@ import java.util.regex.*;
 import java.math.*;
 import paraselene.*;
 import paraselene.tag.*;
-import paraselene.tag.attr.*;
 
 /**
  * 入力部品基底クラス。
@@ -182,7 +181,7 @@ public abstract class Control extends Tag {
 			if ( cont[i].isEmptyValue() )	continue;
 			String	str = cont[i].getValueString();
 			if ( include != null ) {
-				StringBuffer	buf = new StringBuffer();
+				StringBuilder	buf = new StringBuilder();
 				char[]	ch = str.toCharArray();
 				for ( int k = 0; k < ch.length; k++ ) {
 					boolean	flag = true;
