@@ -13,22 +13,12 @@ public class ControlException extends Exception {
 	private static final long serialVersionUID = 1L;
 	private Control[] err = null;
 
-	/**
-	 * コンストラクタ。
-	 * @param mes メッセージ。
-	 * @param cont 単一のエラーコントロール。
-	 */
-	public ControlException( String mes, Control cont ) {
-		super( mes );
-		err = new Control[]{ cont };
-	}
-
 	/**
 	 * コンストラクタ。
 	 * @param mes メッセージ。
 	 * @param cont 複数のエラーコントロール。
 	 */
-	public ControlException( String mes, Control[] cont ) {
+	public ControlException( String mes, Control ... cont ) {
 		super( mes );
 		err = cont;
 	}
