@@ -14,7 +14,6 @@ import javax.servlet.http.*;
 import paraselene.*;
 import paraselene.supervisor.*;
 import paraselene.tag.*;
-import paraselene.tag.attr.*;
 
 class FormSeq {
 	private volatile int seq = 0;
@@ -73,7 +72,7 @@ public class Form extends Tag {
 	 * 削除要素の検証。
 	 * @param tag 削除要素。
 	 */
-	public void checkTag4rm( Tag tag ) {
+	public synchronized void checkTag4rm( Tag tag ) {
 		Tag[]	in_tag = tag.getTagArray();
 		for ( int i = 0; i < in_tag.length; i++ ) {
 			checkTag4rm( in_tag[i] );
@@ -105,7 +104,7 @@ public class Form extends Tag {
 	 * 全てのコントロールの取得。
 	 * @return コントロール配列。存在しない場合、0個の配列を返します。
 	 */
-	public Control[] getAllControl() {
+	public synchronized Control[] getAllControl() {
 		ArrayList<Control>	list = new ArrayList<Control>();
 		for ( String	all_k: control.keySet() ) {
 			HashMap<String,Control>	map = control.get( all_k );
@@ -121,7 +120,7 @@ public class Form extends Tag {
 	 * @param name コントロールのname属性。
 	 * @return コントロール配列。存在しない場合、0個の配列を返します。
 	 */
-	public Control[] getAllControl( String name ) {
+	public synchronized Control[] getAllControl( String name ) {
 		ArrayList<Control>  list = new ArrayList<Control>();
 		HashMap<String,Control>	map = control.get( Page.getIndexName( name ) );
 		if ( map != null ) {
@@ -147,7 +146,7 @@ public class Form extends Tag {
 	 * 追加要素の検証。
 	 * @param tag 追加要素。
 	 */
-	public void checkTag( Tag tag ) {
+	public synchronized void checkTag( Tag tag ) {
 		Tag[]	in_tag = tag.getTagArray();
 		for ( int i = 0; i < in_tag.length; i++ ) {
 			checkTag( in_tag[i] );
@@ -185,7 +184,7 @@ public class Form extends Tag {
 	 * @param req リクエスト。
 	 * @return 変更のあったコントロール。
 	 */
-	public ArrayList<Control> reflect( RequestParameter req ) {
+	public synchronized ArrayList<Control> reflect( RequestParameter req ) {
 		Control[]	cont = getAllControl();
 		ArrayList<Control>	set = new ArrayList<Control>();
 		HashMap<String, CheckBox>	after = new HashMap<String, CheckBox>();
